//+--------------------------------------------------------------------------------------------+
//|                                                                      DrawHistoryOrders.mq4 |
//|                                                             Copyright  2009, Serega Lykov |
//|                                                                 http://mtexperts.narod.ru/ |
//+--------------------------------------------------------------------------------------------+

#property copyright "Copyright  2009, Serega Lykov"
#property link      "http://mtexperts.narod.ru/"

#property show_inputs

//---- external parameters --------------------------------------------------------------------+
extern datetime StartDate           = D'01.06.2009 00:00:00';
extern datetime FinishDate          = D'01.07.2009 00:00:00';
extern int      MagicNumber         = -1;
extern color    BuyOrdersColor      = Blue;
extern color    SellOrdersColor     = Red;
extern bool     DeleteObjects       = false;

//----- global variables ----------------------------------------------------------------------+
static int      CodeArrowOpen       = 1;
static int      CodeArrowClose      = 3;
static double   spread;
static string   symbol;
static string   script_name         = "Script DrawHistoryOrders";
static string   obj_signature       = "DHO";
static string   obj_signature_open  = "_Open ";
static string   obj_signature_close = "_Close ";
static string   obj_signature_trade = "_Trade ";
static string   err_message_1       = "Script can not select an order, error = ";
static string   message_1           = "It is found orders: ";
static string   sign_open;
static string   sign_close;
static string   sign_trade;

//---------------------------------------------------------------------------------------------+
//---- initialization of script ---------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int init()
  {
   Comment(script_name);
   symbol     = Symbol();
   sign_open  = StringConcatenate(obj_signature,obj_signature_open);
   sign_close = StringConcatenate(obj_signature,obj_signature_close);
   sign_trade = StringConcatenate(obj_signature,obj_signature_trade);
   spread     = NormalizeDouble(MarketInfo(symbol,MODE_SPREAD)*Point,Digits);
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- deinitialization of script -------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int deinit()
  {
   Comment("");
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- Script DrawHistoryOrders ---------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
int start()
  {
   DeleteObjects(obj_signature);
   if(DeleteObjects) return(0);
   int num_orders = 0;
   int total = OrdersHistoryTotal();
   for(int i=0; i<total; i++)
     {
      if(!OrderSelect(i,SELECT_BY_POS,MODE_HISTORY))
        {
         Print(StringConcatenate(err_message_1,DoubleToStr(GetLastError(),0)));
         continue;
        }
      if(OrderType() > OP_SELL) continue;
      datetime order_close_time = OrderCloseTime();
      if(order_close_time < StartDate || FinishDate < order_close_time) continue;
      if(MagicNumber >= 0 && OrderMagicNumber() != MagicNumber) continue;
      if(OrderSymbol() != symbol) continue; 
      DrawOrder();
      num_orders++;
     }
   Print(StringConcatenate(message_1,DoubleToStr(num_orders,0)));
   return(0);
  }

//---------------------------------------------------------------------------------------------+
//---- DeleteObjects --------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void DeleteObjects(string obj_sign)
  {
   int obj_total = ObjectsTotal() - 1;
   for(int i=obj_total; i>=0; i--)
     {
      string obj_name = ObjectName(i);
      int obj_type = ObjectType(obj_name);
      if(obj_type != OBJ_ARROW && obj_type != OBJ_TREND) continue;
      if(StringFind(obj_name,obj_sign,0) != 0) continue;
      ObjectDelete(obj_name);
     }
  }

//---------------------------------------------------------------------------------------------+
//---- DrawOrder ------------------------------------------------------------------------------+
//---------------------------------------------------------------------------------------------+
void DrawOrder()
  {
   string str_order_ticket = DoubleToStr(OrderTicket(),0);
   int order_type = OrderType();
   string obj_name = StringConcatenate(sign_open,str_order_ticket);
   if((order_type % 2) == OP_BUY) double order_open_price = NormalizeDouble(OrderOpenPrice() - spread,Digits);
   else order_open_price = NormalizeDouble(OrderOpenPrice(),Digits);
   if(ObjectFind(obj_name) < 0)
     {
      ObjectCreate(obj_name,OBJ_ARROW,0,OrderOpenTime(),order_open_price);
      ObjectSet(obj_name,OBJPROP_STYLE,STYLE_SOLID);
      if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
      else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
      ObjectSet(obj_name,OBJPROP_ARROWCODE,CodeArrowOpen);
     }
   else
     {
      double obj_price1  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
      datetime obj_time1 = ObjectGet(obj_name,OBJPROP_TIME1);
      if(obj_price1 != order_open_price || obj_time1 != OrderOpenTime()) ObjectMove(obj_name,0,OrderOpenTime(),order_open_price);
     }
   obj_name = StringConcatenate(sign_close,str_order_ticket);
   if((order_type % 2) == OP_BUY) double order_close_price = NormalizeDouble(OrderClosePrice(),Digits);
   else order_close_price = NormalizeDouble(OrderClosePrice() - spread,Digits);
   if(ObjectFind(obj_name) < 0)
     {
      ObjectCreate(obj_name,OBJ_ARROW,0,OrderCloseTime(),order_close_price);
      ObjectSet(obj_name,OBJPROP_STYLE,STYLE_SOLID);
      if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
      else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
      ObjectSet(obj_name,OBJPROP_ARROWCODE,CodeArrowClose);
     }
   else
     {
      obj_price1  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
      obj_time1 = ObjectGet(obj_name,OBJPROP_TIME1);
      if(obj_price1 != order_close_price || obj_time1 != OrderCloseTime()) ObjectMove(obj_name,0,OrderCloseTime(),order_close_price);
     }
   obj_name = StringConcatenate(sign_trade,str_order_ticket);
   if(ObjectFind(obj_name) < 0)
     {
      ObjectCreate(obj_name,OBJ_TREND,0,OrderOpenTime(),order_open_price,OrderCloseTime(),order_close_price);
      ObjectSet(obj_name,OBJPROP_STYLE,STYLE_DOT);
      if((order_type % 2) == OP_BUY) ObjectSet(obj_name,OBJPROP_COLOR,BuyOrdersColor);
      else ObjectSet(obj_name,OBJPROP_COLOR,SellOrdersColor);
      ObjectSet(obj_name,OBJPROP_RAY,false);
     }
   else
     {
      obj_price1         = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE1),Digits);
      obj_time1          = ObjectGet(obj_name,OBJPROP_TIME1);
      double obj_price2  = NormalizeDouble(ObjectGet(obj_name,OBJPROP_PRICE2),Digits);
      datetime obj_time2 = ObjectGet(obj_name,OBJPROP_TIME2);
      if(obj_price1 != order_open_price  || obj_time1 != OrderOpenTime())  ObjectMove(obj_name,0,OrderOpenTime(),order_open_price);
      if(obj_price2 != order_close_price || obj_time2 != OrderCloseTime()) ObjectMove(obj_name,1,OrderCloseTime(),order_close_price);
     }
  }

//---------------------------------------------------------------------------------------------+